@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2004 by Puto Moura                                      *
- *   mojo@localhost.localdomain                                            *
+ *   Copyright (C) 2008 by Paulo Moura Guedes                              *
+ *   moura@kdewebdev.org                                                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -15,13 +15,14 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
+ 
 #include "linkchecker.h"
 #include "linkstatushelper.h"
 #include "searchmanager.h"
-#include "../utils/utils.h"
-#include "../parser/htmlparser.h"
+#include "utils/utils.h"
+#include "parser/htmlparser.h"
 #include "klsconfig.h"
 
 #include <QString>
@@ -41,20 +42,26 @@
 #include <dom/dom_html.h>
 
 LinkChecker::LinkChecker(LinkStatus* linkstatus, int time_out,
-                         QObject *parent)
+                         QObject* parent)
         : QObject(parent), search_manager_(0),
         linkstatus_(linkstatus), t_job_(0), time_out_(time_out), document_charset_(),
         redirection_(false), header_checked_(false), finnished_(false), 
         parsing_(false), is_charset_checked_(false), has_defined_charset_(false)
 {
+//     static int i = 0;
     Q_ASSERT(linkstatus_);
 //     Q_ASSERT(!linkstatus_->checked()); // rechecks are now implemented
 
     kDebug(23100) << "Checking " << linkstatus_->absoluteUrl().url();
+
+//     kDebug(23100) << "LinkChecker::LinkChecker " << ++i;
 }
 
 LinkChecker::~LinkChecker()
 {
+//     static int i = 0;
+//     kDebug(23100) << "LinkChecker::~LinkChecker " << ++i;
+    
     delete t_job_;
     t_job_ = 0;
 }
@@ -73,34 +80,41 @@ void LinkChecker::check()
     Q_ASSERT(url.isValid());
 
     if(url.hasRef()) {
-        QTimer::singleShot(0, this, SLOT(slotCheckRef()));
+        KMimeType::Ptr mimeType = KMimeType::findByUrl(url);
+        if(mimeType->is("text/html") || mimeType->is("application/xml")) {
+            QTimer::singleShot(0, this, SLOT(slotCheckRef()));
+            return;
+        }
     }
-    else
-    {
-        t_job_ = KIO::get(url, KIO::NoReload, KIO::HideProgressInfo);        
+        
+    t_job_ = KIO::get(url, KIO::NoReload, KIO::HideProgressInfo);
 //         t_job_->setUiDelegate(0);
 
-        t_job_->addMetaData("PropagateHttpHeader", "true"); // to have the http header
-        if(search_manager_->sendIdentification())
-        {
-            t_job_->addMetaData("SendUserAgent", "true");
-            t_job_->addMetaData("UserAgent", search_manager_->userAgent());
-        }
-        else
-            t_job_->addMetaData("SendUserAgent", "false");
-        
-        
-        QObject::connect(t_job_, SIGNAL(data(KIO::Job *, const QByteArray &)),
-                         this, SLOT(slotData(KIO::Job *, const QByteArray &)));
-        QObject::connect(t_job_, SIGNAL(mimetype(KIO::Job *, const QString &)),
-                         this, SLOT(slotMimetype(KIO::Job *, const QString &)));
-        QObject::connect(t_job_, SIGNAL(result(KJob *)),
-                         this, SLOT(slotResult(KJob *)));
-        QObject::connect(t_job_, SIGNAL(redirection(KIO::Job *, const KUrl &)),
-                         this, SLOT(slotRedirection(KIO::Job *, const KUrl &)));
+    t_job_->addMetaData("PropagateHttpHeader", "true"); // to have the http header
 
-        QTimer::singleShot(time_out_ * 1000, this, SLOT(slotTimeOut()));
+    if (linkstatus_->parent()) {
+        t_job_->addMetaData("referrer", linkstatus_->parent()->absoluteUrl().prettyUrl());
     }
+
+    if(search_manager_->sendIdentification())
+    {
+        t_job_->addMetaData("SendUserAgent", "true");
+        t_job_->addMetaData("UserAgent", search_manager_->userAgent());
+    }
+    else
+        t_job_->addMetaData("SendUserAgent", "false");
+
+
+    QObject::connect(t_job_, SIGNAL(data(KIO::Job *, const QByteArray &)),
+                        this, SLOT(slotData(KIO::Job *, const QByteArray &)));
+    QObject::connect(t_job_, SIGNAL(mimetype(KIO::Job *, const QString &)),
+                        this, SLOT(slotMimetype(KIO::Job *, const QString &)));
+    QObject::connect(t_job_, SIGNAL(result(KJob *)),
+                        this, SLOT(slotResult(KJob *)));
+    QObject::connect(t_job_, SIGNAL(redirection(KIO::Job *, const KUrl &)),
+                        this, SLOT(slotRedirection(KIO::Job *, const KUrl &)));
+
+    QTimer::singleShot(time_out_ * 1000, this, SLOT(slotTimeOut()));
 }
 
 void LinkChecker::slotTimeOut()
@@ -113,12 +127,16 @@ void LinkChecker::slotTimeOut()
         Q_ASSERT(t_job_);
         if(t_job_->error() != KIO::ERR_USER_CANCELED)
         {
+            if(redirection_) {
+                processRedirection(redirection_url_);
+            }
             linkstatus_->setErrorOccurred(true);
-            linkstatus_->setChecked(true);
+//             linkstatus_->setChecked(true);
             linkstatus_->setError("Timeout");
             linkstatus_->setStatus(LinkStatus::TIMEOUT);
 
             killJob();
+            kDebug(23100) << "LinkChecker::slotTimeOut - " << linkstatus_->absoluteUrl().url();
             finnish();
         }
     }
@@ -159,6 +177,7 @@ void LinkChecker::slotMimetype (KIO::Job* /*job*/, const QString &type)
                 ls->setStatus(LinkStatus::SUCCESSFULL);
                 
                 killJob();                
+                kDebug(23100) << "LinkChecker::slotMimetype - " << linkstatus_->absoluteUrl().url();
                 finnish();
             }
         }
@@ -177,6 +196,7 @@ void LinkChecker::slotMimetype (KIO::Job* /*job*/, const QString &type)
                     ls->setStatus(LinkStatus::SUCCESSFULL);
                     
                     killJob();                    
+                    kDebug(23100) << "LinkChecker::slotMimetype - " << linkstatus_->absoluteUrl().url();
                     finnish();
                 }
             }
@@ -205,7 +225,7 @@ void LinkChecker::slotData(KIO::Job* /*job*/, const QByteArray& data)
 
     if(!t_job_->error())
     {
-      if(ls->onlyCheckHeader() && !KLSConfig::showMarkupStatus())
+        if(ls->onlyCheckHeader() && !KLSConfig::showMarkupStatus())
         {
             Q_ASSERT(header_checked_ == false);
             // the job should have been killed in slotMimetype
@@ -222,9 +242,13 @@ void LinkChecker::slotData(KIO::Job* /*job*/, const QByteArray& data)
 
                 if(header_checked_)
                 {
+                    if(redirection_) {
+                        processRedirection(redirection_url_);
+                    }
                     killJob();
                     linkstatus_->setStatus(getHttpStatus());
-                    linkstatus_->setChecked(true);
+//                     linkstatus_->setChecked(true);
+//                     kDebug(23100) << "LinkChecker::slotData 1 - " << linkstatus_->absoluteUrl().url();
                     finnish();
                     return;
                 }
@@ -238,11 +262,18 @@ void LinkChecker::slotData(KIO::Job* /*job*/, const QByteArray& data)
                 {
                     ls->setHttpHeader(getHttpHeader(t_job_));                    
                 }
-                if(ls->mimeType() != "text/html" && header_checked_)
+                if((ls->mimeType() != "text/html" && ls->mimeType() != "application/xml" )
+                    && header_checked_)
                 {
-                    //kDebug(23100) <<  "mimetype of " << ls->absoluteUrl().prettyUrl() << ": " << ls->mimeType();
+                    kDebug(23100) <<  "mimetype of " << ls->absoluteUrl().url() << ": "
+                        << ls->mimeType();
+
                     ls->setStatus(getHttpStatus());
+                    if(redirection_) {
+                        processRedirection(redirection_url_);
+                    }
                     killJob();
+//                     kDebug(23100) << "LinkChecker::slotData 2 - " << linkstatus_->absoluteUrl().url();
                     finnish(); // if finnish is called before kill what you get is a segfault, don't know why
                     return;
                 }
@@ -251,8 +282,12 @@ void LinkChecker::slotData(KIO::Job* /*job*/, const QByteArray& data)
                     //kDebug(23100) <<  "ERROR PAGE";
                     ls->setIsErrorPage(true);
                     ls->setStatus(getHttpStatus());
-                    ls->setChecked(true);
+//                     ls->setChecked(true);
+                    if(redirection_) {
+                        processRedirection(redirection_url_);
+                    }
                     killJob();
+//                     kDebug(23100) << "LinkChecker::slotData 3 - " << linkstatus_->absoluteUrl().url();
                     finnish();
                     return;
                 }
@@ -270,7 +305,7 @@ void LinkChecker::slotData(KIO::Job* /*job*/, const QByteArray& data)
             if(!codec)
                 codec = QTextCodec::codecForName("iso8859-1"); // default
             
-            doc_html_ += codec->toUnicode(data);
+            doc_html_.append(codec->toUnicode(data));
         }
     }
 }
@@ -299,7 +334,7 @@ void LinkChecker::slotResult(KJob* /*job*/)
     if(finnished_)
         return;
 
-    //kDebug(23100) <<  "LinkChecker::slotResult -> " << linkstatus_->absoluteUrl().url();
+    kDebug(23100) <<  "LinkChecker::slotResult -> " << linkstatus_->absoluteUrl().url();
 
     Q_ASSERT(t_job_);
     if(!t_job_)
@@ -308,7 +343,7 @@ void LinkChecker::slotResult(KJob* /*job*/)
     if(redirection_) {
         if(!processRedirection(redirection_url_)) {
             t_job_ = 0;
-            linkstatus_->setChecked(true);
+//             linkstatus_->setChecked(true);
             finnish();
             return;
         }
@@ -324,6 +359,7 @@ void LinkChecker::slotResult(KJob* /*job*/)
         kWarning(23100) << endl << "Job killed quietly, yet signal result was emitted...";
         kDebug(23100) << LinkStatusHelper::toString(linkstatus_);
 
+//         kDebug(23100) << "LinkChecker::slotResult - " << linkstatus_->absoluteUrl().url();
         finnish();
         return;
     }
@@ -349,7 +385,9 @@ void LinkChecker::slotResult(KJob* /*job*/)
 //     if(ls->isErrorPage())
 //         kWarning(23100) << "\n\n" << LinkStatusHelper::toString(ls) << endl;
 
-    Q_ASSERT(!job->isErrorPage());
+    // This can happen, Job Error code:  111
+    // http://www.wi-fi.org/OpenSection/index.asp
+//     Q_ASSERT(!job->isErrorPage());
 
     if(job->error())
     {
@@ -403,24 +441,25 @@ void LinkChecker::slotResult(KJob* /*job*/)
             ls->setDocHtml(doc_html_);
 
             parsing_ = true;
-            HtmlParser parser(doc_html_);
-
-            if(parser.hasBaseUrl())
-                ls->setBaseURI(KUrl(parser.baseUrl().url()));
-            if(parser.hasTitle())
-                ls->setHtmlDocTitle(parser.title().attributeTITLE());
-            
-            ls->setChildrenNodes(parser.nodes());
+//             HtmlParser parser(doc_html_);
+// 
+//             if(parser.hasBaseUrl())
+//                 ls->setBaseURI(KUrl(parser.baseUrl().url()));
+//             if(parser.hasTitle())
+//                 ls->setHtmlDocTitle(parser.title().attributeTITLE());
+//             
+//             ls->setChildrenNodes(parser.nodes());
             parsing_ = false;
         }
     }
+//     kDebug(23100) << "LinkChecker::slotResult - " << linkstatus_->absoluteUrl().url();
     finnish();
 }
 
 void LinkChecker::slotRedirection (KIO::Job* /*job*/, const KUrl &url)
 {
-//     kDebug(23100) <<  "LinkChecker::slotRedirection -> " << 
-//             linkstatus_->absoluteUrl().url()  << " -> " << url.url() << endl;
+    kDebug(23100) <<  "LinkChecker::slotRedirection -> " <<
+            linkstatus_->absoluteUrl().url()  << " -> " << url.url() << endl;
 //             << " - " << t_job_->slave() << "/" <<  t_job_->slave()->slave_pid() << endl;
     
     redirection_ = true;
@@ -432,8 +471,8 @@ bool LinkChecker::processRedirection(KUrl const& toUrl)
     if(finnished_)
         return true;
 
-//     kDebug(23100) <<  "LinkChecker::processRedirection -> " << linkstatus_->absoluteUrl().url() 
-//         << " -> " << toUrl.url() << endl;
+    kDebug(23100) <<  "LinkChecker::processRedirection -> " << linkstatus_->absoluteUrl().url()
+        << " -> " << toUrl.url() << endl;
 
     Q_ASSERT(t_job_);
     Q_ASSERT(linkstatus_->absoluteUrl().protocol().startsWith("http"));
@@ -486,16 +525,20 @@ void LinkChecker::finnish()
 
     if(!finnished_)
     {
-        //kDebug(23100) <<  "LinkChecker::finnish -> " << linkstatus_->absoluteUrl().url();
+        kDebug(23100) <<  "LinkChecker::finnish - " << linkstatus_->absoluteUrl().url();
 
         finnished_ = true;
 
-        if(redirection_)
+        if(redirection_) {
+            // processRedirection already called
             Q_ASSERT(linkstatus_->checked());
-        else
             linkstatus_->setChecked(true);
+        }
+        else {
+            linkstatus_->setChecked(true);
+        }
 
-        emit transactionFinished(linkstatus_, this);
+          emit transactionFinished(linkstatus_, this);
     }
 }
 
@@ -538,6 +581,7 @@ void LinkChecker::slotCheckRef()
     if(ref.isEmpty() || ref == "top") {
         linkstatus_->setStatusText("OK");
         linkstatus_->setStatus(LinkStatus::SUCCESSFULL);
+        kDebug(23100) << "LinkChecker::slotCheckRef - " << linkstatus_->absoluteUrl().url();
         finnish();
         return;
     }
@@ -553,15 +597,15 @@ void LinkChecker::slotCheckRef()
     {
         i_ref = url.url().indexOf('#');
         url_base = url.url().left(i_ref);
-        //kDebug(23100) << "url_base: " << url_base;
+//         kDebug(23100) << "url_base: " << url_base;
 
         Q_ASSERT(search_manager_);
 
         ls_parent = search_manager_->linkStatus(url_base);
     }
 
-    if(ls_parent)
-        checkRef(ls_parent);
+    if(ls_parent && ls_parent->checked() && ls_parent->docHtml().isEmpty())
+          checkRef(ls_parent);
     else
     {
         url = KUrl(url.url().left(i_ref));
@@ -571,15 +615,22 @@ void LinkChecker::slotCheckRef()
 
 void LinkChecker::checkRef(KUrl const& url)
 {
+//     kDebug(23100) << "LinkChecker::checkRef(KUrl const& url)" << url.url();
     Q_ASSERT(search_manager_);
     
     QString url_string = url.url();
     KHTMLPart* html_part = search_manager_->htmlPart(url_string);
     if(!html_part)
     {
-        kDebug(23100) << "new KHTMLPart: " +  url_string;
+//         kDebug(23100) << "new KHTMLPart: " +  url_string;
 
-        html_part = new KHTMLPart();
+        QWidget* w = 0;
+        html_part = new KHTMLPart(w, this);
+//         html_part = new KHTMLPart();
+        html_part->setJScriptEnabled(false);
+        html_part->setJavaEnabled(false);
+        html_part->setMetaRefreshEnabled(false);
+        html_part->setPluginsEnabled(false);
         html_part->setOnlyLocalReferences(true);
 
         QString tmpFile;
@@ -612,36 +663,33 @@ void LinkChecker::checkRef(KUrl const& url)
         linkstatus_->setStatus(LinkStatus::BROKEN);
     }
 
+//     kDebug(23100) << "LinkChecker::checkRef - " << linkstatus_->absoluteUrl().url();
     finnish();
 }
 
 void LinkChecker::checkRef(LinkStatus const* linkstatus_parent)
 {
+//     kDebug(23100) << "LinkChecker::checkRef(LinkStatus const* linkstatus_paren)";
+    
     Q_ASSERT(search_manager_);
 
-    QString url_string = linkstatus_parent->absoluteUrl().url();
-    KHTMLPart* html_part = search_manager_->htmlPart(url_string);
-    if(!html_part)
-    {
-        kDebug(23100) << "new KHTMLPart: " +  url_string;
-
-        html_part = new KHTMLPart();
-        html_part->setOnlyLocalReferences(true);
-
-        html_part->begin();
-        html_part->write(linkstatus_parent->docHtml());
-        html_part->end();
-
-        search_manager_->addHtmlPart(url_string, html_part);
+    bool found = false;
+    
+    QList<Node*> const& children_nodes = linkstatus_parent->childrenNodes();
+    for(int i = 0; i != children_nodes.size(); ++i) {
+        if(children_nodes[i]->element() == Node::A) {
+            NodeA* nodeA = static_cast<NodeA*> (children_nodes[i]);
+            if(nodeA->attributeNAME() == linkStatus()->absoluteUrl().ref()) {
+                found = true;
+            }
+        }
     }
 
-    if(hasAnchor(html_part, linkStatus()->absoluteUrl().ref()))
-    {
+    if(found) {
         linkstatus_->setStatusText("OK");
         linkstatus_->setStatus(LinkStatus::SUCCESSFULL);
     }
-    else
-    {
+    else {
         linkstatus_->setErrorOccurred(true);
         linkstatus_->setError("Link destination not found.");
         linkstatus_->setStatus(LinkStatus::BROKEN);
